#ifndef YERBACON_PARSER_HPP
#define YERBACON_PARSER_HPP

#include <string>
#include "ParseComponents.hpp"
#include "../Yerbacon.hpp"
#include <concepts>

namespace Parser {
    class ParsingException: public Yerbacon::Exception {
        using Yerbacon::Exception::Exception;
    };
    ParseTree parseVector(const vector<tok>& lexed) {
        ParseTree parseTree;
        using namespace StandardComponents;
        using enum tok::type;
        unsigned int i = 0;
        const auto nextAre = [&i, &lexed]<convertible_to<tok::type> T>(const initializer_list<T>& nextValues) {
            unsigned int j = 1;
            for (const T& nextValue: nextValues) {
                if (!cmp_greater(lexed.size() - i, nextValues.size()) || lexed[i + j].toktype != nextValue) {
                    return false;
                }
                ++j;
            }
            return true;
        };
        for (;i < lexed.size(); ++i) {
            const tok& current = lexed[i], next = (i != lexed.size() - 1) ? lexed[i + 1] : tok(UNEXPECTED, string(), current.line);

            switch (current.toktype) {
                case STRING: parseTree << types::String(current.toktext.data()); break;
                case IDENTIFIER: {
                    if (current.toktext == "class") {
                        if (next.toktype == IDENTIFIER) {
                            parseTree << Class(next.toktext); ++i;
                        } else throw ParsingException((not next.toktext.empty()) ? '"' + next.toktext + "\" is not a valid class identifier" : "A class identifier is required", next.line);
                    } else {
                        bool isFinalDefine = nextAre({TAG, DEFINE});
                        if (isFinalDefine || next.toktype == DEFINE) {
                            parseTree << Define(isFinalDefine, current.toktext);
                            i += 1 + isFinalDefine;
                        } else {
                            parseTree << Reference(current.toktext);
                        }
                    }
                }
                default: break;
            }
        }
        return parseTree;
    }
}

#endif //YERBACON_PARSER_HPP